/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <AppKit/AppKit.h>
#import "BindingItem.h"
#import "CORBAObject_Private.h"
#import "IDLDefinitionScope_Private.h"
#import "ORB.h"

@interface NSObject(BindingIterator)
- (void) destroy;
@end

@implementation BindingItem

static double ctxt_timeout = CTXT_TIMEOUT;

NSString* VXInvalidateTableItem = @"VXInvalidateTableItem";

- (IDLInterface*) interface
{
    return [[ORB defaultORB]interfaceNamed:@"CosNaming/NamingContext"];
}

- initReference:(NSString*)idString kind:(NSString*)kind inContext:(CORBAObject*)ctxt
{
    self = [super init];
    _id = [idString retain];
    _kind = [kind retain];
    _ctxt = [ctxt retain];
    _type = 0;
    return self;
}

- initContext:(NSString*)idString kind:(NSString*)kind inContext:(CORBAObject*)ctxt
{
    self = [super init];
    _id = [idString retain];
    _kind = [kind retain];
    _ctxt = [ctxt retain];
    _type = 1;
    return self;
}

- (NSArray*) entries
{
    return _entries;
}

- initRoot:(CORBAObject*)ctxt
{
    self = [super init];
    _ref = [ctxt retain]; 
    _type = 2;
	[_ref setDelegate:self];
    [_ref setInterface:[self interface]]; // typeId may be not supplied
    return self;
}

- (void) dealloc
{
    [[NSNotificationCenter defaultCenter]removeObserver:self];
    [_id release];
    [_kind release];
    [_ctxt release];
    [_ref release];
    [_entries release];
    [super dealloc];
}

- (void) objectDidInvalidate:(NSNotification*)notification
{
	[_ref release];
	_ref = nil;
	if([self isContext]){
		[_entries release];
		_entries = nil;
	}
	[ORB postNotification:[NSNotification notificationWithName:VXInvalidateTableItem object:self]];
}

- (BOOL) object:(CORBAObject*)object willForwardInvocation:invocation toObject:(CORBAObject*)newObject permanently:(BOOL)perm
{
	[_ref release];
	_ref = [newObject retain];
	[_ref setDelegate:self];
	[_ref setInterface:[self interface]]; // typeId may be not supplied
	[ORB postNotification:[NSNotification notificationWithName:VXInvalidateTableItem object:self]];
	return YES; 
}

- (BOOL) isContext
{
    return (_type != 0);
}

- (NSArray*) _name
{
    NSMutableDictionary* dict = [NSMutableDictionary dictionary];
    NSArray* array = [NSArray arrayWithObject:dict];
    if(_id)
        [dict setObject:_id forKey:@"id"];
    if(_kind)
        [dict setObject:_kind forKey:@"kind"];
    return array;
}

- (void) fetch
{
    @try{
		int cnt = [[NSUserDefaults standardUserDefaults]integerForKey:@"nameServiceFetchCount"];
		NSArray* array = nil;
		id iterator = nil;
		if(cnt <= 0)
			cnt = 100;
		[[IDLDefinitionScope rootScope]set]; // Set the current scope
		[_ref setReplyTimeout:ctxt_timeout];
		[_ref list:cnt bindingList:&array bindingIterator:&iterator];
		if([array count] > 0){
			NSEnumerator* en = [array objectEnumerator];
			id item;
			if(!_entries)
				_entries = [[NSMutableArray alloc]init];
			else
				[_entries removeAllObjects];
			while(item = [en nextObject]){
				int type = [[item valueForKey:@"binding_type"]intValue];
				NSArray* nameArray = [item valueForKey:@"binding_name"];
				id nameComponent = [nameArray objectAtIndex:0];
				NSString* idString = [nameComponent valueForKey:@"id"];
				NSString* kind = [nameComponent valueForKey:@"kind"];
				BindingItem* b;
				if(type == 0)
					b = [[BindingItem allocWithZone:[self zone]]
						initReference:idString kind:kind inContext:_ref];
				else
					b = [[BindingItem allocWithZone:[self zone]]
						initContext:idString kind:kind inContext:_ref];
				[_entries addObject:[b autorelease]];
			}
		}
		if(iterator)
			[iterator destroy]; // Destroy BindingIterator
    }@catch(NSException *localException){
		[localException raise];
    }
    
}

- (void) resolve
{
    @try{
		if(_ctxt && !_ref){
			[[IDLDefinitionScope rootScope]set]; // Set the current scope
			_ref = [[_ctxt resolve:[self _name]]retain];
			[_ref setDelegate:self];
			if([self isContext]){
				[_ref setInterface:[self interface]]; // typeId may be not supplied
				[self fetch];
			}
		}
    }@catch(NSException *localException){
		NSLog(@"%@: %@\n",
			[localException name], [localException reason]);
		[localException raise];
    }
}

- (BOOL) hasValidInterface
{
    if([self isContext])
        return _ref != nil && [_ref interface] != nil && [_ref implementsInterfaceNamed:@"NamingContext"];
    else
        return _ref != nil && [_ref interface] != nil;
}

- (BOOL) isResolved
{
    return _ref != nil;
}

- (NSString*) object
{
    return [self isContext]? @"CTXT" : @"IOR";
}

- (NSString*) objectTypeId
{
    return [_ref valueForKey:@"type_id"];
}

- outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:item
{
    if(item)
        return [[item entries]objectAtIndex:index];
    else
        return [[self entries]objectAtIndex:index];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:item
{
    return [item isContext] && [item isResolved];
}

- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
    if(item)
        return [[item entries]count];
    else
        return [[self entries]count];
}

- outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:item
{
    return [item valueForKey:[tableColumn identifier]];
}

extern NSString* CORBAIORPboardType;

- (BOOL) writeToPasteboard:(NSPasteboard*)pboard
{
    if(_ref){
        NSArray* types = [NSArray arrayWithObjects:CORBAIORPboardType, NSStringPboardType, nil];
        NSString* ior = [_ref toString];
        [pboard declareTypes:types owner:nil];
        [pboard setString:ior forType:CORBAIORPboardType];
        [pboard setString:ior forType:NSStringPboardType];
        return YES;
    }else
        return NO;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView writeItems:(NSArray*)items toPasteboard:(NSPasteboard*)pboard
{
    if([items count] > 0){
        BindingItem* item = [items objectAtIndex:0];
        if([item isResolved] && ![item isContext])
            return [item writeToPasteboard:pboard];
    }
    return NO;
}

- handleQueryWithUnboundKey:(NSString*)key
{
    NSLog(@"[valueForKey]: unboundKey: %@\n", key);
    return @"???";
}

- connection
{
    return _ref;
}

@end
