/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "IDLDefinition_Private.h"
#import "IDLInterface_Private.h"
#import "ObjectCell.h"
#import "IORView.h"
#import "IDLViewer.h"

@implementation IDLInterface(Viewer)

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"interface13"];
}

- (void) loadCell:cell
{
    [cell setLoaded:YES];
    [cell setLeaf:NO];
	[cell setImage:[self cellImage]];
    [cell setRepresentedObject:self];
}

- (NSString*) title
{
    return [self name];
}

- (NSString*) category
{
    return @"- Interfaces -";
}

- (NSDictionary*) _getCategories
{
    NSEnumerator* en = [_list keyEnumerator];
    NSString* key;
    NSMutableDictionary* dict = [NSMutableDictionary dictionary];

    while(key = [en nextObject]){
        IDLDefinition* def = [_list objectForKey:key];
        NSString* category = [def category];
        if(category){
            NSMutableDictionary* cdict;
            if(!(cdict = [dict objectForKey:category]))
                [dict setObject:cdict = [NSMutableDictionary dictionary] forKey:category];
            [cdict setObject:def forKey:key];
        }
    }
    return dict;
}

- (void) fillMatrix:matrix
{
    NSDictionary* dict = [self _getCategories];
    NSArray* categories = [[dict allKeys]sortedArrayUsingSelector:@selector(compare:)];
    NSEnumerator* catEn = [categories objectEnumerator];
    NSString* cat;
    
    if([_inheritance count] > 0){
        NSEnumerator* en = [_inheritance objectEnumerator];
        int n = [matrix numberOfRows];
        IDLInterface* def;
        id cell;
        
        while(def = [en nextObject]){
            [matrix addRow];
            cell = [matrix cellAtRow:n++ column:0];
            [cell setStringValue:[def name]];
            [def loadCell:cell];
        }
    }
    
    while(cat = [catEn nextObject])
        [[dict objectForKey:cat]fillMatrix:matrix];
}

- (NSString*) stringValue
{
    NSString* string = [NSString stringWithFormat:@"%@interface %@",
        _isAbstract?@"abstract ":(_isLocal?@"local ":@""),
        [self name]];
    if([_inheritance count] > 0){
        NSString* delimiter = @": ";
        NSEnumerator* en = [_inheritance objectEnumerator];
        IDLInterface* ifObj;
        while(ifObj = [en nextObject]){
            string = [string stringByAppendingFormat:@"%@%@", delimiter,[ifObj name]];
            delimiter = @", ";
        }
    }
    if([[self typeId] hasPrefix:@"IDL:"] && [[self version]length] > 0)
        string = [string stringByAppendingFormat:@" <%@>", [self version]];
    return string;
}

- (BOOL) writeDefinitionToPboard:(NSPasteboard*)pboard
{
    if([self isAbstract] || [self isLocal])
        return NO;
    else
        return [super writeDefinitionToPboard:pboard];
}

@end

