/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import "IDLDefinition_Private.h"
#import "IDLSequence.h"
#import "IDLArray.h"
#import "IDLOctetArray.h"
#import "IDLPrimitive.h"
#import "OctetArrayFormatter.h"
#import "IDLOctetSequence.h"
#import "OctetSequenceFormatter.h"
#import "ObjectCell.h"
#import "IDLViewer.h"

@implementation IDLSequence(Viewer)

- (NSString*) title
{
    return [type title];
}

- (NSString*) stringValue
{
    if(_frameCount > 0)
        return [NSString stringWithFormat:@"sequence <%@,%d>",[type stringValue],_frameCount];
    else
        return [NSString stringWithFormat:@"sequence <%@>",[type stringValue]];
}

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"array13"];
}

- (void) loadCell:cell
{
    [type loadCell:cell];
    [cell setRepresentedObject:self];
	[cell setImage:[self cellImage]];
}

- (void) fillMatrix:matrix
{
    [type fillMatrix:matrix];
}


- (unsigned) frameCount
{
    return _frameCount;
}
- (void) setFrameCount:(unsigned)count
{
    _frameCount = count;
}

@end

@implementation IDLArray(Viewer)

- (NSString*) stringValue
{
    return [NSString stringWithFormat:@"%@[%d]", [[self arrayType]stringValue], _size];
}

- (NSString*) title
{
    return [_type title];
}

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"array13"];
}

- (void) loadCell:cell
{
    [_type loadCell:cell];
    [cell setRepresentedObject:self];
	[cell setImage:[self cellImage]];
}

- (void) fillMatrix:matrix
{
    [_type fillMatrix:matrix];
}

@end

@implementation IDLOctetArray(Viewer)

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"octet13"];
}

- (void) loadCell:cell
{
    [cell setLoaded:YES];
    [cell setLeaf:YES];
    [cell setRepresentedObject:self];
	[cell setImage:[self cellImage]];
}

- (void) fillMatrix:matrix
{
}

@end

@implementation IDLOctetSequence(Viewer)

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"octet13"];
}

- (void) loadCell:cell
{
    [cell setLoaded:YES];
    [cell setLeaf:YES];
    [cell setRepresentedObject:self];
	[cell setImage:[self cellImage]];
}

- (void) fillMatrix:matrix
{
}

@end

