/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import "IDLDefinition_Private.h"
#import "IDLUnion.h"
#import "IDLSwitchCase.h"
#import "ObjectCell.h"
#import "IDLViewer.h"

@implementation IDLUnion(Viewer)

- (NSString*) title
{
    return @"switch";
}

- (NSString*) stringValue
{
    return [NSString stringWithFormat:@"union %@ switch(%@)",
        [self name],[_type name]];
}

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"union13"];
}

- (void) loadCell:cell
{
    [cell setLoaded:YES];
    [cell setLeaf:NO];
    [cell setRepresentedObject:self];
	[cell setImage:[self cellImage]];
}

- (void) fillMatrix:matrix
{
    NSEnumerator* enumerator = [_caseList objectEnumerator];
    IDLSwitchCase* obj;
    int n = 0;
    id cell;

    while(obj = [enumerator nextObject]){
        NSString* string = [obj caseStringValue];
        [matrix addRow];
        [cell = [matrix cellAtRow:n++ column:0]setStringValue:string?string:@"<undefined>"];
        [obj loadCell:cell];
    }
    if(_defaultCase){
        [matrix addRow];
        [cell = [matrix cellAtRow:n++ column:0]setStringValue:@"default"];
        [_defaultCase loadCell:cell];
    }
}

@end

@implementation IDLSwitchCase(Viewer)

- (NSImage*) cellImage
{
	return [_type cellImage];
}
- (void) loadCell:cell
{
    [_type loadCell:cell];
    [cell setLoaded:YES];
    [cell setRepresentedObject:self];
}

- (void) fillMatrix:matrix
{
    [_type fillMatrix:matrix];
}

- (NSString*) stringValue
{
    if(!_value)
        return [NSString stringWithFormat:@"default: %@ %@",
                [_type stringValue], _name];
    else
        return [NSString stringWithFormat:@"case %@: %@ %@",
                [self caseStringValue], [_type stringValue], _name];
}

@end

