/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <Cocoa/Cocoa.h>
#import "IDLDefinition.h"
#import "IDLValueType.h"

@class CORBAObject;

@interface IDLItem : NSObject 
{
	IDLDefinition *_type;
	NSString* _name;
	id _value;
	id _controller;
	struct {
		unsigned editable:1;
		unsigned itemsLoaded:1;
		unsigned observing:1;
		unsigned notInTree:1;
		unsigned:28;
	} _flags;
}

- initWithIDLType:(IDLDefinition *)def;
+ itemWithIDLType:(IDLDefinition *)def;
- (void) setItemName:(NSString *)name;
- (NSString *)itemName;
- itemValue;
- (void) setItemValue:value;
- value;
- (void) setValue:value;
- (IDLDefinition*)itemType;
- controller;
- (void)setController:controller;
- (NSCell *) dataCell;
- (BOOL) isLeaf;
- (NSArray *) items;
- (unsigned) itemCount;
- (BOOL) isItemEditable;
- (BOOL) isEditable;
- (void) setEditable:(BOOL)flag;
- (void) resetCell:(NSTextFieldCell *)cell;
- (IDLInterface *)interfaceObject;
- (BOOL) shouldSetIDLType:(IDLDefinition *)type;
- (BOOL) shouldInsertItemWithIDLType:(IDLDefinition *)type;
- (void) setIDLType:(IDLDefinition *)def;
- insertNewItemWithIDLType:(IDLDefinition *)def atIndex:(int)index;
- (BOOL) shouldTakeValueFromItem:(IDLItem *)item;
- (BOOL) shouldInsertItem:(IDLItem *)item;
- (void) takeValueFromItem:(IDLItem *)item;
- (void) insertItem:(IDLItem *)item atIndex:(int)index;
- (NSString *) valueDescription;
+ (NSColor *) classColor;
- (BOOL) writeToPasteboard:(NSPasteboard *)pboard;
- (void) startObservingValueChanges;
- (void) stopObservingValueChanges;
- (BOOL) isInTree;
- (void) setIsInTree:(BOOL)flag;
@end

extern NSString *IDLItemPboardType, *IDLItemLinkPboardType;

@interface IDLBooleanItem:IDLItem
@end

@interface IDLOctetItem:IDLItem
@end

@interface IDLNumberItem:IDLItem
@end

@interface IDLStringItem:IDLItem
@end
@interface IDLEnumItem:IDLItem
@end

@interface IDLStructItem:IDLItem
{
	NSMutableArray *_items;
}
@end

@interface IDLValueClassItem:IDLStructItem

@end

@interface IDLArrayItem:IDLItem
{
	NSMutableArray *_items;
}
@end

@interface IDLSequenceItem:IDLItem
{
	NSMutableArray *_items;
}
- insertNewItemAtIndex:(int)index;
- (void) removeItemAtIndex:(int)index;
- (BOOL) shouldInsertNewItem;
- (BOOL) shouldRemoveItem;
@end

@interface IDLFunctionItem:IDLItem
{
	NSMutableArray *_items;
}
@end

@interface IDLIORItem:IDLItem
- (BOOL) takeValueFromPasteboard:(NSPasteboard *)pboard forType:(NSString *)type;
@end

@interface IDLValueItem:IDLItem
{
	IDLItem *_valueItem;
	id _valueType;
}
- (void) clearItemValue;
- (void) resetItemValue;
- (void) setItemValue;
- (void) setItemType:(IDLDefinition*)type;
@end

@interface IDLAbstractItem:IDLValueItem

@end

@interface IDLAnyItem:IDLItem
{
	IDLItem *_valueItem;
}

@end

@interface IDLUnionItem:IDLItem
{
	IDLItem *_descriptorItem;
	IDLItem *_valueItem;
}
@end

@interface IDLCharItem:IDLItem

@end

@interface IDLAttributeItem:NSObject
{
	NSString *_name;
	IDLDefinition *_type;
	IDLItem *_valueItem;
}
- initWithIDLType:(IDLDefinition*)type;
+ itemWithIDLType:(IDLDefinition*)type;
- (void) setValue:value;
- value;
@end

@interface IDLDefinition(IDLItem)
- (Class) itemClass;
- (BOOL) isLeaf;
- (BOOL) isItemEditable;
- (NSCell *) itemDataCell;
- defaultItemValue;
- defaultItemValueForIDLType:(IDLDefinition*)type;
- defaultValueItemValue;
- wrapperValueForValue:value withIDLType:(IDLDefinition *)type;
- (IDLItem *) createIDLItemWithName:(NSString *)name;
- (IDLItem*) createIDLItemWithName:(NSString*)name forIDLType:(IDLDefinition*)type;
- (void) resetItemCell:(NSTextFieldCell *)cell;

@end

@interface IDLDefinition(IDLItemItems)
- (NSMutableArray *) itemArrayForItem:(IDLItem *)item;
- (void) addValueObserver:(IDLItem*)structItem forValue:value;
- (void) removeValueObserver:(IDLItem*)structItem forValue:value;
@end
@interface IDLValueType(IDLItem)
- (IDLItem *) createValueItemWithName:(NSString *)name;
- defaultValueItemValue;
- (unsigned) memberCount;
- (NSArray *) itemArrayForItem:(IDLItem *)item;
@end
@interface IDLAttribute(IDLItem)
- (IDLItem *) createValueItemWithName:(NSString *)name;
@end
@interface IDLEnumCell:NSPopUpButtonCell
@end
@interface IORCell:NSTextFieldCell
@end
@interface IORFormatter:NSFormatter
{
	int _mode;
}
- (int) formatMode;
- (void) setFormatMode:(int)mode;
@end

@interface IDLCharFormatter:NSFormatter
@end

@interface IDLArrayObject:NSObject <NSCoding>
{
	NSMutableArray* _value;
}
+ array;
+ arrayWithArray:(NSArray*)array;
- init;
- initWithArray:(NSArray*)array;
- (void) addObject:object;
@end

