/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "NamingTestClient.h"
#import <ADORB/ADORB.h>
#import "NamingContextProto.h"

@interface DSize:NSObject
{
    double _width;
    double _height;
}
- initWidth:(double)w height:(double)h;
+ size:(double)w :(double)h;
@end

@interface XSize:NSObject
{
    DSize* _size;
    NSArray* _ref;
}
- initSize:(DSize*)size ref:(NSArray*)ref;
+ xsize:(DSize*)size ref:(NSArray*)ref;
@end

@implementation NamingTestClient

- init
{
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    CORBAObject* namingService = [ORB resolveInitialReferences:@"NameService"];
    self = [super init];
    if(namingService){
        NSString* subPath = [defaults stringForKey:@"bind"];
        NSString* path = @"test/numbers";
        if(subPath)
            path = [subPath stringByAppendingPathComponent:path];
        @try{
			server = [[namingService resolve:[IDLNameComponent nameComponentsForPath:path]]retain];
        }@catch(NSException *localException){
			NSLog(@"%@: %@, info = %@\n", 
				[localException name], [localException reason], [[localException userInfo]description]);
        }
		return self;
    }else{
		NSLog(@"NamingTestClient: No naming service, exiting");
        [self release];
        return nil;
    }
}

- (void) dealloc
{
    [server release];
    [super dealloc];
}

- (void) run
{
	NSLog(@"%@: starting...\n", [self className]);
    @try{
		NSDictionary* size = [DSize size:3.0:4.0];
		NSArray* sizes = [NSArray arrayWithObject:size];
		NSArray* diags;
		IDLAnyObject* any;
		NSDictionary* t, *s;
		id def = [[ORB defaultORB]definitionNamed:@"ORBTest/TestAny"];
		
		NSArray* numbers = [server _getNumbers];
		NSLog(@"Server returned: numbers = %@\n", [numbers componentsJoinedByString:@", "]);
		
		diags = [server diag:sizes];
		NSLog(@"Server returned: diags = %@\n", [diags componentsJoinedByString:@", "]);
		
		t = [NSDictionary dictionaryWithObjectsAndKeys:
			[NSNumber numberWithInt:1], IDLUnionDescriptorKey,
			[NSNumber numberWithDouble:5], IDLUnionDataKey, nil];
		s = [NSDictionary dictionaryWithObjectsAndKeys:
			t, @"alias",
			[NSNumber numberWithInt:1], @"idx",
			[NSArray arrayWithObjects:@"1",@"2",@"3",nil], @"desc", nil];
		any = [IDLAnyObject objectWithIDLType:def];
		[any setValue:s];
		any = [server copyAny:any];
		NSLog(@"copy_any: server returned\n%@", [any description]);
		
		// TestIndir
		any = [IDLAnyObject objectWithIDLType:[[ORB defaultORB]definitionNamed:@"ORBTest/TestIndir"]];
		[any setValue:[XSize xsize:[DSize size:3:4] 
				ref:[NSArray arrayWithObjects:
				[XSize xsize:[DSize size:3:8] ref:nil], [XSize xsize:[DSize size:4:8] ref:nil], nil]]];
		
		any = [server copyAny:any];
		NSLog(@"copy_any: server returned\n%@", [any description]);
		
    }@catch(NSException *localException){
		NSLog(@"Exception %@: %@\n", [localException name], [localException reason]);
    }
}

@end

@implementation DSize
- initWidth:(double)w height:(double)h
{
    self = [super init];
    _width = w;
    _height = h;
    return self;
}
- init
{
    return [self initWidth:0.0 height:0.0];
}

+ size:(double)w :(double)h
{
    return [[[self alloc]initWidth:w height:h]autorelease];
}

- (NSString*)description
{
    return [NSString stringWithFormat:@"(%f,%f)", _width,_height];
}

@end

@implementation XSize

- initSize:(DSize*)size ref:(NSArray*)ref
{
    self = [super init];
    _size = [size retain];
    _ref = [ref retain];
    return self;
}
+ xsize:(DSize*)size ref:(NSArray*)ref
{
    return [[[self alloc]initSize:size ref:ref]autorelease];
}
- (void) dealloc
{
    [_ref release];
    [_size release];
    [super dealloc];
}
- (NSString*)description
{
    return [NSString stringWithFormat:@"XSize %@ ref (%@)",
        _size, ([_ref count] > 0)?[_ref description]:@"none"];
}
@end
