/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLAbstractBase.h"
#import "IDLInterface_Private.h"
#import "IDLDefinition_Private.h"
#import "IDLEncapsulation.h"

@implementation IDLAbstractBase

static id _instance = nil;

- init
{
    self = [super initWith:nil inherited:nil];
    [self setName:@"AbstractBase"];
    [self setIsAbstract:YES];
    return self;
}

+ sharedInstance
{
    if(!_instance)
        _instance = [[self allocWithZone:[self zone]]init];
    return _instance;
}

- (long) kind
{
    return tk_native;
}

- (BOOL) adoptsInterface:(IDLInterface*)pifObj
{
    return NO;
}

- (NSString*)stringValue
{
    return @"native AbstractBase";
}

- (NSString*) description
{
    return [[super description] stringByAppendingString:@"native AbstractBase"];
}

- (void) encodeWithCORBACoder:(id <CORBACoder>)coder
{
    [super encodeWithCORBACoder:coder];
    [coder encodeString:[self typeId]];
    [coder encodeString:[self name]];
}

+ _definitionFromCoder:(IDLEncapsulation*)coder
{
    [NSException raise:CORBANotImplemented format:@"demarshaling typecode for AbstractBase is not implemented"];
    return nil;
}

- (id)awakeAfterUsingCoder:(NSCoder *)aDecoder
{
	if(_instance){
		[self release];
		return _instance;
	}else
		return _instance = self;
}

@end
