/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLDefinition_Private.h"
#import "IDLAttribute_Private.h"
#import "IDLInterface_Private.h"
#import "IDLPrimitive.h"
#import "IDLFunction_Private.h"
#import "IDLValueType.h"
#import "IDLArgument.h"
#import <expr.tab.h>

@implementation IDLAttribute

- (void) setReadOnly:(BOOL)flag
{
    _isReadOnly = flag?YES:NO;
}

- (BOOL) isReadOnly
{
    return _isReadOnly;
}

- initWithType:(IDLDefinition*)type name:(NSString*)name
{
    self = [super init];
    _name = [name retain];
    _type = [type retain];
    return self;
}
    
+ (IDLAttribute*) attributeWithType:(IDLDefinition*)type andName:(NSString*)name
{
    return [[[self allocWithZone:[self zone]]initWithType:type name:name]autorelease];
}

- (void) dealloc
{
    [_type release];
    [_name release];
    [_get release];
    [_set release];
    [super dealloc];
}

- (IDLDefinition*) attributeType
{
    return _type;
}

- (NSString*) name
{
    return _name;
}

- (void) setInterfaceObject:(IDLInterface*) anObj
{
    _interfaceObject = anObj;
}

- (IDLInterface*)interfaceObject
{
    return _interfaceObject;
}

- (IDLFunction*)getAttribute
{
    if(!_get){
        _get = [[IDLFunction function:[NSString stringWithFormat:@"_get_%@",_name]
                    returning:_type
                    args:[NSArray array]]retain];
        [_get setInterfaceObject:_interfaceObject];
    }
    return _get;
}

- (IDLFunction*)setAttribute
{
    if(!_isReadOnly && !_set){
        IDLArgument* arg = [IDLArgument argumentWithName:_name type:_type mode:IN];
        _set = [[IDLFunction function:[NSString stringWithFormat:@"_set_%@",_name]
                returning:[[IDLDefinitionScope rootScope] definitionNamed:@"void"]
                args:[NSArray arrayWithObject:arg]]retain];
        [_set setInterfaceObject:_interfaceObject];
    }
    return _set;
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	[super encodeWithCoder:coder];
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_type forKey:@"type"];
		[coder encodeObject:_interfaceObject forKey:@"interface"];
		[coder encodeObject:_name forKey:@"name"];
		[coder encodeObject:_get forKey:@"get"];
		[coder encodeObject:_set forKey:@"set"];
		[coder encodeBool:_isReadOnly forKey:@"readOnly"];
	}else{
		[coder encodeObject:_type];
		[coder encodeObject:_interfaceObject];
		[coder encodeObject:_name];
		[coder encodeObject:_get];
		[coder encodeObject:_set];
		[coder encodeValueOfObjCType:@encode(BOOL) at:&_isReadOnly];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super initWithCoder:coder];
	if([coder allowsKeyedCoding]){
		_type = [[coder decodeObjectForKey:@"type"]retain];
		_interfaceObject = [[coder decodeObjectForKey:@"interface"]retain];
		_name = [[coder decodeObjectForKey:@"name"]retain];
		_get = [[coder decodeObjectForKey:@"get"]retain];
		_set = [[coder decodeObjectForKey:@"set"]retain];
		_isReadOnly = [coder decodeBoolForKey:@"readOnly"];
	}else{
		_type = [[coder decodeObject]retain];
		_interfaceObject = [[coder decodeObject]retain];
		_name = [[coder decodeObject]retain];
		_get = [[coder decodeObject]retain];
		_set = [[coder decodeObject]retain];
		[coder decodeValueOfObjCType:@encode(BOOL) at:&_isReadOnly];
	}
	return self;
}

@end
