/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLDefinitionScope.h"

@protocol IDLScopePrivate <IDLDefinitionScope>

- (void) addDefinition:(id <IDLDefinition>)def withName:(NSString*)name;
- addDefinition:(id <IDLDefinition>)def withIdentifierArray:(NSArray*)identArray;
- (void) setParentScope:(id<IDLDefinitionScope>)scope;
- (void) setName:(NSString*)name;
- (NSDictionary*)_definitionDictionary;
- (void) _setVersion:(NSString*)version forDefinitionNamed:(NSString*)defName;
- (void) set;

@end

@interface IDLDefinitionScope(Private) <IDLScopePrivate>

- init;
- initRootScope;
+ (IDLDefinitionScope*) definitionScope;
+ (IDLDefinitionScope*) definitionScopeWithParentScope:(id<IDLDefinitionScope>)scope;
+ (id <IDLDefinitionScope>)definitionScopeWithName:(NSString*)name;

- (id <IDLDefinition>) _definitionNamed:(NSString*)name;
+ (void) _setCurrentPrefix:(NSString*) string;

- (void) setName:(NSString*)name;

- (void) awakeFromParse;

- (NSArray*)allKeys;
- (NSEnumerator*) objectEnumerator;
- (NSEnumerator*) keyEnumerator;

+ (void) pushScope:(IDLDefinitionScope*)scope;
+ (IDLDefinitionScope*)popScope;
+ (void) clearScopeStack;

- (void) setNamePath:(NSString*)path;
+ (void) _setCurrentPrefix:(NSString*) string;
+ (NSString*) currentPrefix;
- (void) applyVersions;

@end

@interface IDLDefinitionScope(Pragma)
// pragma and special commands support
+ (void) performCommand:command with:(NSArray*)values;

@end
