/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>
#import "IDLDefinition_Private.h"
#import "IDLFunction_Private.h"
#import "IDLArgument.h"
#import "IDLInterface_Private.h"
#import "IDLValueType.h"
#import "IDLException.h"
#import "CORBAException.h"
#import "ORB.h"
#import <expr.tab.h>
#import <objc/runtime.h>

@interface VXWrapper:NSObject
{
    void* _ptr;
}
- (void*)pointerReference;
+ wrapper;
@end

@interface NSMethodSignature(_private)
+ signatureWithObjCTypes:(const char*)types;
@end

struct _infoP{
    long offset;
    long size;
    const char* type;
    void* reserved1;
    void* reserved2;
};

#if (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_5) && defined(__ppc__)
static int _frame_offset = marg_prearg_size;
//static int _frame_max_base = marg_prearg_size + 32;
#endif

@implementation IDLFunction(CORBACoding)

- (unsigned) returnValueLength
{
    return [[self returnType]returnValueLength];
}

- (NSString*) _objCTypes
{
    NSMutableString* str = [NSMutableString stringWithUTF8String:[[self returnType]objCType]];
	[str appendFormat:@"%s%s", @encode(id), @encode(SEL)];
    NSEnumerator* en = [_args objectEnumerator];
    IDLArgument* def;
    while(def = [en nextObject]){
        if([def mode] != IN)
			[str appendFormat:@"^%s", [[def definition]objCType]];
		else
			[str appendFormat:@"%s", [[def definition]objCType]];
	}
    if(_contextList)
		[str appendFormat:@"%s", @encode(id)];
	
	return str;
}

#if (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_5) && defined(__ppc__)
- (void) _createMethodSignature
{
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    struct _shadow { @defs(NSMethodSignature) } *ms;
    NSMutableString* str = [NSMutableString stringWithUTF8String:[[self returnType]objCType]];
    NSEnumerator* en = [_args objectEnumerator];
    int cnt = [_args count];
    int n,l;
    int s = 1;
    int offset = _frame_offset;
    int offset8 = 0;
    unsigned long addr0;
    IDLArgument* def;
    const char* argType;
    struct _infoP* pi;
    
    if(_contextList)
        cnt += 1;
    ms = _signature = NSZoneMalloc([self zone], sizeof(struct _shadow));
    ms->isa = [NSMethodSignature class];
    ms->_fixup = NULL;
    ms->_reserved = NULL;
    ms->_parmInfoP = pi = _data = NSZoneMalloc([self zone],(cnt+2)*sizeof(struct _infoP));
    ms->_nargs = cnt + 2;
    ms->_returnValueLength = [self returnValueLength];
    ms->_sizeofParams = sizeof(id) + sizeof(SEL) + _frame_offset;
    
    [str appendString:@"@:"];
    for(n=0;n<cnt+2;n++)
        pi[n].reserved1 = pi[n].reserved2 = NULL;
    pi[0].offset = offset;
    pi[0].size = 4;
    pi[0].type = (void*)1;
    pi[1].offset = pi[0].offset + 4;
    pi[1].size = 4;
    pi[1].type = (void*)2;
    s = 1;
    n = 2;
    offset += pi[1].size + pi[0].size;
    
    while(def = [en nextObject]){
        BOOL isPtr = ([def mode] != IN);
        char *mod = isPtr?"^" : "";
        argType = [[def definition]objCType];
        pi[n].type = pi[n-1].type + s;
        s = isPtr?2:1;
        [str appendFormat:@"%s%s",mod,argType];
        if(isPtr){
            pi[n].offset = offset;
            offset += (pi[n].size = sizeof(void*));
            ms->_sizeofParams += sizeof(void*);
        }else{
            l = [[def definition]argLength];
            pi[n].size = l;
            if(l <= sizeof(int)){
                pi[n].offset = offset;
                offset += sizeof(int);
                ms->_sizeofParams += sizeof(int);
            }else
            if(argType[0] == 'd'){
                pi[n].offset = offset8;
                pi[n].size = sizeof(double);
                offset8 += sizeof(double);
                offset += sizeof(double);
                ms->_sizeofParams += sizeof(double);
			}else
			if(argType[0] == 'f'){
                pi[n].offset = offset8;
                pi[n].size = sizeof(double);
                offset8 += sizeof(double);
				//if(offset < _frame_max_base){ // as in NSMethodSignature 
					offset += sizeof(float);
					ms->_sizeofParams += sizeof(float);
				//}else{
				//	offset += sizeof(double);
				//	ms->_sizeofParams += sizeof(double);
				//}
            }else{
                ms->_sizeofParams += l;
                pi[n].offset = offset;
                offset += l;
            }
        }
        n += 1;
    }
    if(_contextList){
        pi[n].offset = offset;
        pi[n].type = pi[n-1].type +s;
        [str appendString:@"@"];
        ms->_sizeofParams += sizeof(void*);
        pi[n].size = sizeof(void*);
    }
    
    addr0 = (unsigned long)[str cString];
    for(n=0;n<cnt+2;n++)
        pi[n].type = (void*)((unsigned long)pi[n].type + addr0);
    _objCTypeString = [str retain];
    ms->_types = [_objCTypeString cString];
    ms->_sizeofParams += 15 - (ms->_sizeofParams+15)%16;

    [pool release];
}
#endif

- (NSMethodSignature*)methodSignature
{
#if (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_5) && defined(__ppc__)
    NSLock* lock = [self getLock];
    [lock lock];
    if(!_signature)
        [self _createMethodSignature];
    [lock unlock];
    return (NSMethodSignature*)_signature;
#else
    NSLock* lock = [self getLock];
    [lock lock];
	if(!_signature)
		_signature = [[NSMethodSignature signatureWithObjCTypes:[[self _objCTypes]cStringUsingEncoding:NSASCIIStringEncoding]]retain];
    [lock unlock];
    return (NSMethodSignature*)_signature;	
#endif
}

- (NSMutableDictionary*)getArgValues:(NSMutableDictionary*) dict fromInvocation:(NSInvocation*)inv
{
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    NSException* exception = nil;

    @try{
		NSEnumerator* en = [_args objectEnumerator];
		NSMethodSignature* ms = [inv methodSignature];
		IDLArgument* def;
		int n = 1;
		id argObj;
		void* ptr;

		if(!dict)
			dict = [NSMutableDictionary dictionary];

		while(def = [en nextObject]){
			NSString* argName = [def name];
			n += 1;
			switch([def mode]){
				case IN:
					argObj = [[def definition] argumentObjectFromInvocation:inv atIndex:n];
					if(argObj)
						[dict setObject:argObj forKey:argName];
					break;
				case INOUT:
					[inv getArgument:&ptr atIndex:n];
					if(ptr == NULL)
						[NSException raise:CORBAInvalidArgumentException 
							format:@"NULL pointer for INOUT argument %@", argName];
					argObj = [[def definition] argumentObjectOfType:[ms getArgumentTypeAtIndex:n]
								fromPointer:ptr];
					if(argObj)
						[dict setObject:argObj forKey:argName];
					break;
				default:
					break;
			}
		}
		if(_contextList){
			NSArray* array = nil;
			[inv getArgument:&array atIndex:n+1];
			[dict setValue:array forKey:@"$context"]; // context may be nil
		}

    }@catch(NSException *localException){
		exception = [localException retain];
    }

    [dict retain];
    [pool release];
    [dict autorelease];
    if(exception)
        [[exception autorelease]raise];
    return dict;
}

// Set result values for invocation after invoking method remotely (client side)
- (void) setValues:(NSDictionary*)dict forInvocation:(NSInvocation*)inv
{
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    NSException* exception = nil;
    NSEnumerator* en = [_args objectEnumerator];
    NSMethodSignature* ms = [inv methodSignature];
    IDLArgument* def;
    int n = 1;
    id argObj;
    void* ptr;

    @try{
		while(def = [en nextObject]){
			NSString* argName = [def name];
			n += 1;
			switch([def mode]){
				case INOUT:
				case OUT:
					[inv getArgument:&ptr atIndex:n];
					if(ptr == NULL)
						[NSException raise:CORBAInvalidArgumentException 
							format:@"NULL pointer for INOUT/OUT argument %@", argName];
					argObj = [[[dict objectForKey:argName]retain]autorelease];
					[[def definition]setValue:argObj ofType:[ms getArgumentTypeAtIndex:n]
							forPointer:ptr];
					break;
				default:
					break;
			}
		}
    }@catch(NSException *localException){
		exception = [localException retain];
    }
    [pool release];
    if(exception)
        [[exception autorelease]raise];
    
    argObj = [[[dict objectForKey:@"$return"]retain]autorelease];
    [[self returnType]setReturnValue:argObj forInvocation:inv];
}

// Encodes args given in the dictionary 'obj'
- (void) encodeObject:obj withCoder:(id <CORBACoder>)coder
{
    NSEnumerator* en = [_args objectEnumerator];
    IDLArgument* def;

    while(def = [en nextObject]){
        NSString* argName = [def name];
        switch([def mode]){
            case IN:
            case INOUT:
                // Should we use 'valueForKey:' ?
                [[def definition]encodeObject:[obj objectForKey:argName] withCoder:coder];
                break;
            default:
                break;
        }
    }
    if(_contextList){
        id context = [obj objectForKey:@"$context"];
        [self _encodeContextList:context withCoder:coder];
    }
}

// Decode args to dictionary 'obj'
// 'obj' is the existing dictionary; check if it is mutable first
- (void) decodeObject:(id*)obj withCoder:(id <CORBADecoder>)coder
{
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    NSException* exception = nil;
    
    NSEnumerator* en = [_args objectEnumerator];
    IDLArgument* def;
    id value = nil;

    @try{
		if(*obj == nil)
			*obj = [NSMutableDictionary dictionary];
		else{
			if(![*obj isKindOfClass:[NSMutableDictionary class]])
				*obj = [NSMutableDictionary dictionaryWithDictionary:*obj];
		}
		[*obj retain];
		
		[[self returnType]decodeObject:&value withCoder:coder];
		[*obj setValue:value forKey:@"$return"];
		
		while(def = [en nextObject]){
			NSString* argName = [def name];
			switch([def mode]){
				case OUT:
				case INOUT:
					value = nil;
					[[def definition]decodeObject:&value withCoder:coder];
					[*obj setValue:value forKey:argName];
					break;
				default:
					break;
			}
		}
    }@catch(NSException *localException){
		exception = [localException retain];
    }
    [pool release];
    
    [*obj autorelease];
    if(exception)
        [[exception autorelease]raise];
}

// Decode request's args to dictionary 'obj'
// 'obj' is the existing dictionary; check if it is mutable first
- (void) decodeRequest:(id*)obj withCoder:(id <CORBADecoder>)coder
{
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    NSException* exception = nil;
    NSEnumerator* en = [_args objectEnumerator];
    IDLArgument* def;
    id value = nil;
    BOOL rc = YES;

    @try{
		if(*obj == nil)
			*obj = [NSMutableDictionary dictionary];
		else{
			if(![*obj isKindOfClass:[NSMutableDictionary class]])
				*obj = [NSMutableDictionary dictionaryWithDictionary:*obj];
		}
		[*obj retain];
		
		while(rc && (def = [en nextObject])){
			NSString* argName = [def name];
			switch([def mode]){
				case IN:
				case INOUT:
					value = nil;
					[[def definition]decodeObject:&value withCoder:coder];
					[*obj setValue:value forKey:argName];
					break;
				default:
					break;
			}
		}
		if(_contextList){
			id context = nil;
			[self _decodeContextList:&context withCoder:coder];
			[*obj setValue:context forKey:@"$context"];
		}
    }@catch(NSException *localException){
		exception = [localException retain];
    }
    [pool release];
    [*obj autorelease];
    if(exception)
        [[exception autorelease]raise];
}

- (void) encodeReply:obj withCoder:(id <CORBACoder>)coder
{
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    NSException *exception = nil;
    @try{
		NSEnumerator* en = [_args objectEnumerator];
		IDLArgument* def;
		id value = [obj objectForKey:@"$return"];

		[[self returnType]encodeObject:value withCoder:coder];

		while(def = [en nextObject]){
			NSString* argName = [def name];
			switch([def mode]){
				case OUT:
				case INOUT:
					value = [obj objectForKey:argName];
					[[def definition]encodeObject:value withCoder:coder];
					break;
				default:
					break;
			}
		}
    }@catch(NSException *localException){
		exception = [localException retain];
    }
    [pool release];
    if(exception)
        [[exception autorelease]raise];
}

// Returns invocation only if the selector exists
- (NSInvocation*) invocation
{
    NSString* selectorName = [self selectorName];
    SEL selector = NSSelectorFromString(selectorName);
    if(selector){
        NSInvocation* inv = [NSInvocation invocationWithMethodSignature:[self methodSignature]];
        [inv setSelector:selector];
        return inv;
    }else
        return nil;
}

- (void) _setupInvocation:(NSInvocation*)inv withArguments:(NSMutableDictionary*)dict
{
    if(inv){
        NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
        NSException* exception = nil;
        
        @try{
			NSMutableDictionary* values = [NSMutableDictionary dictionary];
			NSEnumerator* en = [_args objectEnumerator];
			NSMethodSignature* ms = [inv methodSignature];
			IDLArgument* def;
			int n = 1;
			id argObj;
			void** ptr;
			VXWrapper* wrapper;
			
			while(def = [en nextObject]){
				NSString* argName = [def name];
				n += 1;
				switch([def mode]){
					case IN:
						argObj = [dict objectForKey:argName];
						[[def definition]setValue:argObj forInvocation:inv atIndex:n];
						break;
					case INOUT:
						argObj = [dict objectForKey:argName];
						wrapper = [VXWrapper wrapper];
						ptr = [wrapper pointerReference];
						[[def definition]setValue:argObj ofType:[ms getArgumentTypeAtIndex:n]
								forPointer:ptr];
						[inv setArgument:&ptr atIndex:n];
						[values setObject:wrapper forKey:argName];
						break;
					case OUT:
						wrapper = [VXWrapper wrapper];
						ptr = [wrapper pointerReference];
						*ptr = NULL;
						[inv setArgument:&ptr atIndex:n];
						[values setObject:wrapper forKey:argName];
						break;
					default:
						break;
				}
			}
			if(_contextList){
				id context = [dict objectForKey:@"$context"];
				[inv setArgument:&context atIndex:n+1];
			}

			[dict setObject:values forKey:@".values"];
        }@catch(NSException *localException){
			exception = [localException retain];
        }
        [pool release];
        if(exception)
            [[exception autorelease]raise];
    }
}

- (NSInvocation*) invocationWithArguments:(NSMutableDictionary*)dict methodSignature:(NSMethodSignature*)ms
{
    NSInvocation* inv = [NSInvocation invocationWithMethodSignature:ms];
    [self _setupInvocation:inv withArguments:dict];
    return inv;
}

- (NSInvocation*) invocationWithArguments:(NSMutableDictionary*)dict
{
    NSInvocation* inv = [self invocation];
    [self _setupInvocation:inv withArguments:dict];
    return inv;
}

// Exctract result values after invoking the invocation (server side)
- (NSMutableDictionary*)takeValues:(NSMutableDictionary*) dict fromInvocation:(NSInvocation*)inv
{
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    NSException* exception = nil;
    
    @try{
		NSEnumerator* en = [_args objectEnumerator];
		NSMethodSignature* ms = [inv methodSignature];
		IDLArgument* def;
		int n = 1;
		id argObj;
		void* ptr;

		if(!dict)
			dict = [NSMutableDictionary dictionary];

		while(def = [en nextObject]){
			NSString* argName = [def name];
			n += 1;
			switch([def mode]){
				case OUT:
				case INOUT:
					[inv getArgument:&ptr atIndex:n];
					if(ptr == NULL)
						[NSException raise:CORBAInvalidArgumentException
							format:@"NULL pointer for INOUT or OUT argument %@ in '%@'",
							argName,[self name]];
					argObj = [[def definition] argumentObjectOfType:[ms getArgumentTypeAtIndex:n]
								fromPointer:ptr];
					if(argObj)
						[dict setObject:argObj forKey:argName];
					break;
				default:
					break;
			}
		}

		argObj = [[self returnType]returnValueFromInvocation:inv];
		if(argObj)
			[dict setObject:argObj forKey:@"$return"];
    }@catch(NSException *localException){
		exception = [localException retain];
    }
    [dict retain];
    [pool release];
    [dict autorelease];
    if(exception)
        [[exception autorelease]raise];
    return dict;
}

- (IDLException*) exceptionWithTypeID:(NSString*)typeId
{
    // typeID always specifies the full path - so look in the root scope 
    return (IDLException*)[[_interfaceObject rootScope]definitionForTypeId:typeId];
}

- (BOOL) willSendReference
{
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    BOOL rc = NO;
    NSEnumerator* en = [_args objectEnumerator];
    IDLArgument* def;

    while(def = [en nextObject]){
        switch([def mode]){
            case IN:
            case INOUT:{
                IDLDefinition* definition = [def definition];
                if([definition respondsToSelector:@selector(willSendReference)] && 
                        (rc = [definition willSendReference]))
                    break;
            }
        }
    }
    [pool release];
    return rc;
}

#pragma mark ----- Context support -----

static BOOL _match_ctx(NSString* pattern, NSString* name)
{
    if(pattern && name){
        const char* p = [pattern UTF8String];
        const char* n = [name UTF8String];
        while(*p && *n){
            if(*p == '*')
                return YES;
            if(*p != *n)
                return NO;
            p += 1;
            n += 1;
        }
        if(*p == *n)
            return YES;
    }
    return NO;
}

- (NSArray*) _getContextProperties:context
{
// Not optimized
    NSMutableArray* array = [NSMutableArray array];
    NSEnumerator* en = [context objectEnumerator];
    id prop;
    while(prop = [en nextObject]){
        NSString* propName = [prop valueForKey:@"name"];
        NSEnumerator* cen = [_contextList objectEnumerator];
        NSString* ctx;
        while(ctx = [cen nextObject])
            if(_match_ctx(ctx,propName)){
                [array addObject:prop];
                break;
            }
    }
    return array;
}

- (void) _encodeContextList:context withCoder:(id <CORBACoder>)coder
{
    if([_contextList count] > 0){
        NSArray* list = [self _getContextProperties:context];
        long count = [list count]*2;
        NSEnumerator* en = [list objectEnumerator];
        id property;
        [coder encodeLong:count];
        while(property = [en nextObject]){
            [coder encodeString:[property valueForKey:@"name"]];
            [coder encodeString:[property valueForKey:@"value"]];
        }
    }
}

- (void) _decodeContextList:(id*)context withCoder:(id <CORBADecoder>)coder
{
    if([_contextList count] > 0 && [coder length] > [coder location]){
        long count = 0;
        [coder decodeLong:&count];
        if(count%2 == 0){
            NSMutableArray* array = *context = [NSMutableArray array];
            while(count > 0){
                NSString* propName = nil;
                NSString* value = nil;
                [coder decodeString:&propName];
                [coder decodeString:&value];
                [array addObject:[NSDictionary dictionaryWithObjectsAndKeys:
                        propName,@"name", value, @"value", nil]];
                count -= 2;
            }
        }
    }else{
        *context = nil;
        [NSException raise:CORBAMarshalException format:@"failed decoding context"];
    }
}

@end

@implementation VXWrapper

static NSZone* _zone = NULL;

+ (void) initialize
{
    if([VXWrapper class] == self){
        _zone = NSCreateZone(4096,4096,YES);
        NSSetZoneName(_zone,@"WrapperZone");
    }
}
- (void*)pointerReference
{
    return _ptr;
}
- init
{
    self = [super init];
    _ptr = NSZoneMalloc(_zone,16);
    return self;
}

- (void) dealloc
{
    NSZoneFree(_zone,_ptr);
    [super dealloc];
}

+ wrapper
{
    return [[[self alloc]init]autorelease];
}

@end

