/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLTypedValue.h"
#import "IDLDefinition_Private.h"
#import "IDLEnum.h"

@implementation IDLTypedValue

- (void) setVisible:(BOOL) flag
{
    _isHidden = flag?NO:YES;
}

- initWithType:(IDLDefinition*)type value:(NSObject*)value
{
    self = [super init];
    _type = [type retain];
    _value = [[_type coerceValue:value]retain];
    return self;
}

+ (IDLTypedValue*) valueOfType:(IDLDefinition*)type withValue:(NSObject*)value
{
    return [[[self allocWithZone:[self zone]]initWithType:type value:value]autorelease];
}

- (void) deallocate
{
    [_type release];
    [_value release];
    [super dealloc];
}

-(NSString*) description
{
    return [[super description] stringByAppendingFormat:@"const %@ = %@",
            [_type description],[_value description]];
}

- (id <IDLDefinition>)valueType
{
    return _type;
}

- (NSString*) name
{
    return [_type name];
}

static const char* intString = "cCsSiIlLqQ";
//static const char* floatString = "fd";

static BOOL isInt(id number)
{
    const char* type = [number objCType];
    if(strchr(intString,type[0]))
        return YES;
    else
        return NO;
}

- _value
{
    return _value;
}

- (NSString*) valueDescription
{
    return [_type valueDescriptionOf:_value];
}

- value
{
    return [_type convertValue:_value];
}

+ (NSNumber*) evaluate:(int)op with:arg1:arg2
{
    if([arg1 isKindOfClass:[NSNumber class]] && [arg2 isKindOfClass:[NSNumber class]]){
        BOOL bothAreInt = isInt(arg1) && isInt(arg2);
        switch(op){
            case '+':
                if(bothAreInt)
                    return [NSNumber numberWithLong:
                        [arg1 longValue] + [arg2 longValue]];
                else
                    return [NSNumber numberWithDouble:
                        [arg1 doubleValue] + [arg2 doubleValue]];
            case '-':
                if(bothAreInt)
                    return [NSNumber numberWithLong:
                        [arg1 longValue] - [arg2 longValue]];
                else
                    return [NSNumber numberWithDouble:
                        [arg1 doubleValue] - [arg2 doubleValue]];
            case '*':
                if(bothAreInt)
                    return [NSNumber numberWithLong:
                        [arg1 longValue] * [arg2 longValue]];
                else
                    return [NSNumber numberWithDouble:
                        [arg1 doubleValue] * [arg2 doubleValue]];
            case '/':
                if(bothAreInt){
                    long d2 = [arg2 longValue];
                    if(d2 != 0)
                        return [NSNumber numberWithLong:
                            [arg1 longValue]/d2];
                    else{
                        NSLog(@"Division by 0 in expression\n");
                        return nil;
                    }
                }else{
                    NSNumber* r = nil;
                    @try{
						double d2 = [arg2 doubleValue];
						if(d2 != 0.0)
							r = [NSNumber numberWithDouble:
								[arg1 doubleValue]/d2];
						else
							NSLog(@"Division by 0.0 in expression\n");
                    }@catch(NSException *localException){
                        NSLog(@"%@:%@\n",
                            [localException name], [localException reason]);
                        r = nil;
                    }
                    return r;
                }
            case '%':
                if(bothAreInt){
                    long d2 = [arg2 longValue];
                    if(d2 != 0)
                        return [NSNumber numberWithLong:
                            [arg1 longValue] % d2];
                    else{
                        NSLog(@"Division by 0 in expression\n");
                        return nil;
                    }
                }else{
                    NSLog(@"Float operands in '%' expression\n");
                    return nil;
                }
            case '|':
                if(bothAreInt)
                    return [NSNumber numberWithBool:
                        [arg1 boolValue] || [arg2 boolValue]];
                else{
                    NSLog(@"Float operands in '|' expression\n");
                    return nil;
                }
            case '^':
                if(bothAreInt)
                    return [NSNumber numberWithBool:
                        [arg1 boolValue] ^ [arg2 boolValue]];
                else{
                    NSLog(@"Float operands in '^' expression\n");
                    return nil;
                }
            case '&':
                if(bothAreInt)
                    return [NSNumber numberWithBool:
                        [arg1 boolValue] && [arg2 boolValue]];
                else{
                    NSLog(@"Float operands in '&' expression\n");
                    return nil;
                }
            default:
                NSLog(@"Unknown binary operation '%c'\n", op);
                return nil;
        }
    }else{
        NSLog(@"Invalid operands in expression: '%@ %c %@'\n",
            [arg1 description], op, [arg2 description]);
        return nil;
    }
}

+ (NSNumber*) shiftLeft:arg1 by:arg2
{
    if([arg1 isKindOfClass:[NSNumber class]] && [arg2 isKindOfClass:[NSNumber class]]){
        if(isInt(arg1) && isInt(arg2))
            return [NSNumber numberWithLong:
                ([arg1 longValue]<<[arg2 longValue])];
        else{
            NSLog(@"Float operands in '<<' expression\n");
            return nil;
        }
    }else{
        NSLog(@"Invalid operands in expression '%@ << %@'\n",
            [arg1 description], [arg2 description]);
        return nil;
    }
}

+ (NSNumber*) shiftRight:arg1 by:arg2
{
    if([arg1 isKindOfClass:[NSNumber class]] && [arg2 isKindOfClass:[NSNumber class]]){
        if(isInt(arg1) && isInt(arg2))
            return [NSNumber numberWithLong:
                ([arg1 longValue]>>[arg2 longValue])];
        else{
            NSLog(@"Float operands in '>>' expression\n");
            return nil;
        }
    }else{
        NSLog(@"Invalid operands in expression'%@ >> %@'\n",
            [arg1 description], [arg2 description]);
        return nil;
    }
}

+ (NSNumber*) evaluate:(int)op with:arg1
{
    if([arg1 isKindOfClass:[NSNumber class]]){
        switch(op){
            case '+':
                return arg1;
            case '-':
                if(isInt(arg1))
                    return [NSNumber numberWithLong:-[arg1 longValue]];
                else
                    return [NSNumber numberWithDouble:[arg1 doubleValue]];
            case '~':
                if(isInt(arg1))
                    return [NSNumber numberWithBool:
                        [arg1 boolValue]?NO:YES];
                else{
                    NSLog(@"Float argument in unary '~'\n");
                    return nil;
                }
            default:
                NSLog(@"Unknown unary operation '%c'\n", op);
                return nil;
        }
    }else{
        NSLog(@"Invalid argument in unary operation '%c%@'\n",
            op,[arg1 description]);
            return nil;
    }
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	[super encodeWithCoder:coder];
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_type forKey:@"type"];
		[coder encodeObject:_value forKey:@"value"];
		[coder encodeBool:_isHidden forKey:@"hidden"];
	}else{
		[coder encodeObject:_type];
		[coder encodeObject:_value];
		[coder encodeValueOfObjCType:@encode(BOOL) at:&_isHidden];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super initWithCoder:coder];
	if([coder allowsKeyedCoding]){
		_type = [[coder decodeObjectForKey:@"type"]retain];
		_value = [[coder decodeObjectForKey:@"value"]retain];
		_isHidden = [coder decodeBoolForKey:@"hidden"];
	}else{
		_type = [[coder decodeObject]retain];
		_value = [[coder decodeObject]retain];
		[coder decodeValueOfObjCType:@encode(BOOL) at:&_isHidden];
	}
	return self;
}

@end
