/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>
#import "IDLDefinition_Private.h"
#import "IDLValueClass.h"
#import "IDLValueType_Private.h"
#import "IDLDefinitionScope.h"
#import "IDLEncapsulation.h"
#import "IDLValueMember.h"
#import "IDLLink.h"

@implementation IDLValueClass(CORBACoding)

static BOOL logEnabled = NO;

+ (void) initialize
{
    if([IDLValueClass class] == self){
        NSString* debug = [[NSUserDefaults standardUserDefaults]stringForKey:@"debug"];
        if(debug){
            NSRange r = [debug rangeOfString:@"V"];
            if(r.length > 0)
                logEnabled = YES;
        }
    }
}

- (void) _decodeObjectState:(id*)obj withCoder:(id <CORBADecoder>)coder
{
    NSEnumerator* en = [_members objectEnumerator];
    IDLValueMember* member;
	int pos0 = 0;
    while(member = [en nextObject]){
        NSString* name = [member name];
        IDLDefinition* def = [member memberType];
        id value = nil;
		if(logEnabled)
			pos0 = [coder cursorPosition];
        [def decodeObject:&value withCoder:coder];
		@try{
			[*obj setValue:value forKey:name];
			if(logEnabled)
				NSLog(@"value-class: decoded '%@ = 0x%x' for '0x%x' range %d.%d\n", 
					name, value, *obj, pos0, [coder cursorPosition]-pos0);
		}@catch(NSException *localException){
			if([[localException name]isEqualToString:NSUndefinedKeyException]){
				ORBLogUndefinedKeyError(_valuetype, *obj, name);
				if(![IDLDefinition allowsUndefinedKeys])
					[localException raise];
			}else
				[localException raise];
		}
    }
}

- (void) decodeObjectState:(id*)obj withCoder:(id <CORBADecoder>)coder
{
    [_super decodeObjectState:obj withCoder:coder];
    [self _decodeObjectState:obj withCoder:coder];
}

- (void) _encodeObjectState:obj withCoder:(id<CORBACoder>)coder
{
    NSEnumerator* en = [_members objectEnumerator];
    IDLValueMember* member;
    while(member = [en nextObject]){
        NSString* name = [member name];
        IDLDefinition* def = [member memberType];
		id value = nil;
		@try{
			value = [obj valueForKey:name];
			if(logEnabled)
				NSLog(@"value-class: encoding %@ = 0x%x for 0x%x\n", name, value, obj);
		}@catch(NSException *localException){
			if([[localException name]isEqualToString:NSUndefinedKeyException]){
				ORBLogUndefinedKeyError(_valuetype, obj, name);
				if(![IDLDefinition allowsUndefinedKeys])
					[localException raise];
			}else
				[localException raise];
		}
        [def encodeObject:value withCoder:coder];
    }
}

- (void) encodeObjectState:obj withCoder:(id<CORBACoder>)coder
{
    [_super encodeObjectState:obj withCoder:coder];
    [self _encodeObjectState:obj withCoder:coder];
}

- (void) _encodeWithCORBACoder:(id <CORBACoder>)coder
{
    NSEnumerator* en = [_members objectEnumerator];
    IDLValueMember* member;
    [coder encodeLong:[_members count]];
    while(member = [en nextObject]){
        [coder encodeString:[member name]];
        [[member memberType]encodeWithCORBACoder:coder];
        [coder encodeShort:[member accessMode]];
    }
}

- (void) encodeWithCORBACoder:(id <CORBACoder>)coder
{
    [coder encodeShort:_modifier];
    if(_super)
        [_super encodeWithCORBACoder:coder];
    else
        [coder encodeLong:tk_null];
    
    [self _encodeWithCORBACoder:coder];
}

- (void) decodeFromCoder:(id <CORBADecoder>)coder
{
    id base;
    long count;
    [coder decodeShort:&_modifier];
    base = [IDLDefinition definitionFromCoder:coder];
    if([base isKindOfClass:[IDLLink class]])
        _super = [[base link]retain];
    else
        _super = [base retain];
    
    [coder decodeLong:&count];
    while(count-- > 0){
        NSString* str;
        IDLDefinition* def;
        IDLValueMember* member;
        short access;
        [coder decodeString:&str];
        def = [IDLDefinition definitionFromCoder:coder];
        [coder decodeShort:&access];
        member = [IDLValueMember valueMemberOfType:def mode:access name:str];
        [_members addObject:member];
    }
}

@end
